;==============================================================================
; (c) Copyright Elect Software International Inc., 1992, Toronto. Anyone can
; use this code for anything as long as it is not resold as a software
; development resource, as long as the copyright notice isn't removed, as
; long as changes are clearly marked as to authorship, and as long as users
; indemnify Elect from any liability.
; Comments welcome. Henrik Bechmann, CIS:72701,3717; Tel:416-534-8176.
;==============================================================================

;==============================================================================
;                          EVENT DEMO DESCRIPTION
;==============================================================================

; This event demo features an event tracer, and places a variety of windows on
; the desktop together with a window that lists events that take place as
; you interact with the desktop. Hit [AltM] (or right click anywhere but 
; the event tracer window, or on the EVENTS.TXT window if loaded) to get a
; menu, and try out the menu selections. This is NOT a closed application,
; but rather a framework which is purposely as passive as possible, to get
; as close as possible to the native Paradox 4.0 desktop interface.
; Therefore running into problems is as legitimate as watching the events
; tick by in the trace window.

; The only exception to the passivity rule is that [F9], [AltF9], and [F2]
; are intercepted and ExecEvent-ed when in Wait, to give you a chance to see 
; how editing behaves under the Wait command. A message is displayed to 
; indicate this programmatic intervention.

; Of particular interest is the ability to switch rapidly between GetEvent
; and Wait control, to be able to compare and contrast the differences.

; See the contents of Edemo.hlp for more information about the menus.

; The fundamental purpose of this event demo is to test and demonstrate the
; generic event manager EventMan (in the accompanying Eventman.sc file). 
; EventMan is designed to be used in any application that requires 
; sophisticated and systematic event handling facilities.

; See also the Tracer.SC source code file for the Tracer code.

; There are two events that are screened out by the Tracer, because they
; would generate too much "noise": IDLE events, and mouse events with no
; buttons depressed.

;============================================================================
;                           EVENT DEMO INTERFACE
;============================================================================

; EventDemo.Constructor()
; EventDemo.Destructor()

; EventDemo.Run()
; EventDemo.DispatchEventTag() ; intervene at highlevel for keystroke handling

;==============================================================================
;                           EVENT DEMO IMPLEMENTATION
;==============================================================================

Proc EventDemo.Constructor()
   Private
      i
   ;-------------------------------------------------------------------------
   ; Load the generic event manager EventMan, the tracer, and the walking
   ; man...
   ;-------------------------------------------------------------------------
   Librarian.HasControl = True
   Play "EventMan"
   Play "Tracer"
   Play "Walking"
   Release vars Librarian.HasControl
   EventMan.Constructor()
   EventMan.DispatchEventTagProc = "EventDemo.DispatchEventTag"
   Tracer.Constructor()
   ;-------------------------------------------------------------------------
   ; Initialize variables...
   ;-------------------------------------------------------------------------
   EventDemo_TestFileSpec = "Test.txt"
   EventDemo_IsDragAndDrop = False
   
   EventDemo_HelpFileSpec = "Edemo.hlp"
   EventDemo_HelpWindowIsActive = False
   
EndProc ; EventDemo.Constructor

Proc EventDemo.Destructor()
   Release Vars
      EventDemo_TestFileSpec,
      EventDemo_IsDragAndDrop,
      EventDemo_DefaultProcBag,
      EventDemo_WalkingProcBag,
      EventDemo_HelpFileSpec,
      EventDemo_HelpWindowHandle,
      EventDemo_HelpWindowIsActive,
      EventDemo_HelpProcBag
      
   EventMan.Destructor()
EndProc ; EventDemo.Destructor

; --------------------------------- RUN DEMO -------------------------------

Proc EventDemo.Run()
   Private
      EventDemo_IsRunning,
      EventDemo_Mode,
      ReturnCode

   Echo Normal
   
   Message "Loading desktop..."      
   EventDemo!OpenImages()
   EventDemo!OpenTestEditor()
   EventDemo!OpenDragCanvas()
   Tracer.OpenEventTrace()
   EventDemo!OpenEventHelp()
   EventDemo!SetDefaultProcs()
   While True
      Message "Select mode for demo..."
      ShowPopup "Select mode" Centered
         "Wait ~W~orkspace":"Run under wait workspace":"Workspace",
         "Wait ~T~able":"Run under wait table":"Table",
         "Wait ~R~ecord":"Run under wait record":"Record",
         "Wait ~F~ield":"Run under wait field":"Field",
         "~G~etEvent":"Run under GetEvent":"GetEvent"
      EndMenu
      To EventDemo_Mode
      If EventDemo_Mode = "Esc" Then
         Message "Quitting."
         ReturnCode = EventDemo!CloseApp()
         If ReturnCode = 2 Then
            Message "Event demo is over."
            Quitloop
         Else
            Message ""
         Endif
      Else
         EventDemo!SetPulldown()
         Quitloop
      Endif
   EndWhile
   Message ""
   EventDemo_IsRunning = True
   While EventDemo_IsRunning
      Switch
         Case EventDemo_Mode = "Esc":
            EventDemo_IsRunning = False
         Case EventDemo_Mode = "GetEvent":
            Echo Normal
            EventMan.DoGetEvent()
         Case EventDemo_Mode = "Workspace":
            Echo normal
            Wait WorkSpace
               Proc "EventMan.CatchTrigger" All
            EndWait
         Case EventDemo_Mode = "Table":
            Echo Normal
            Wait Table 
               Proc "EventMan.CatchTrigger" All
            EndWait
         Case EventDemo_Mode = "Record":
            Echo Normal
            Wait Record
               Proc "EventMan.CatchTrigger" All
            EndWait
         Case EventDemo_Mode = "Field":
            Echo Normal
            Wait Field
               Proc "EventMan.CatchTrigger" All
            EndWait
      EndSwitch
   EndWhile
   Message "Exiting demo..."
EndProc ; EventDemo.Run

;------------------------- EDEMO.HLP WINDOW ------------------------------

Proc EventDemo!OpenEventHelp()
   Private
      WindowBag
      
   If IsFile(EventDemo_HelpFileSpec) Then
      Editor Open EventDemo_HelpFileSpec
   Else
      Message EventDemo_HelpFileSpec," not found. Creating empty file..."
      Editor New EventDemo_HelpFileSpec
   Endif
   EventDemo_HelpWindowHandle = GetWindow()
   EventDemo_HelpWindowIsActive = True

   Dynarray WindowBag[]
   WindowBag["Title"] = "Event demo help"
   WindowBag["Width"] = 38
   WindowBag["Height"] = 10
   WindowBag["OriginRow"] = 13
   WindowBag["OriginCol"] = 41
   Window SetAttributes EventDemo_HelpWindowHandle From WindowBag

   Dynarray EventDemo_HelpProcBag[]
   EventDemo_HelpProcBag["Close"] = "EventDemo!CloseEventHelp"
   EventDemo_HelpProcBag["Key"] = "EventDemo!BrowseKeycodeHandler"
   EventDemo_HelpProcBag[Asc("F2")] = "EventDemo!CloseEventHelp"
   EventDemo_HelpProcBag["Up"] = "EventDemo!CallControlMenu"
   EventDemo_HelpProcBag["RightDown"] = "EventDemo!HandleRightDown"
   EventDemo_HelpProcBag["ObjectTag"] = StrVal(EventDemo_HelpWindowHandle)
   EventMan.SetHandlersFrom(EventDemo_HelpProcBag)
   
   Return 1
EndProc ; EventDemo!OpenEventHelp

Proc EventDemo!CloseEventHelp()
   Private
      CurrentWindow
   CurrentWindow = GetWindow()
   Window Select EventDemo_HelpWindowHandle
   Window Close
   EventDemo_HelpWindowIsActive = False
   EventMan.ClearHandlersWith(EventDemo_HelpProcBag)
   If IsWindow(CurrentWindow) Then
      Window Select CurrentWindow
   Endif
   Return 1
EndProc ; EventDemo!CloseEventHelp

Proc EventDemo!BrowseKeycodeHandler()
   Private
      Keycode
   Keycode = EventMan.EventBag["Keycode"]
   If (Keycode < 0 or Keycode = 5) ; CtrlE
      And
      (Keycode <> -32 and Keycode <> -83) ; [AltD], [Del]
      Then
      EventMan.EventTag = StrVal(EventMan.EventBag["Keycode"])
      EventMan.DispatchEventTag()
      Return EventMan.ReturnCode
   Else
      Return 1 ; suppress the keystroke
   Endif
EndProc ; EventDemo!BrowseKeycodeHandler

;-------------------------- TEST.TXT WINDOW ---------------------------------

Proc EventDemo!OpenTestEditor()
   Private
      WindowBag,
      WindowHandle
   If IsFile(EventDemo_TestFileSpec) Then
      Editor Open EventDemo_TestFileSpec
   Else
      Editor New EventDemo_TestFileSpec
   Endif
   WindowHandle = GetWindow()
   Dynarray WindowBag[]
   WindowBag["Width"] = 35
   WindowBag["Height"] = 5
   WindowBag["OriginRow"] = 7
   WindowBag["OriginCol"] = 42
   WindowBag["Title"] = "Test Editor"
   Window SetAttributes WindowHandle From WindowBag
EndProc ; EventDemo!OpenTestEditor

;------------------------------ IMAGE WINDOWS -------------------------------

Proc EventDemo!OpenImages()
  Menu {Tools}{Info}{Inventory}{Files} Enter
  EventDemo!SetImageWindow(1,1)
  Menu {Ask} Select table()
  EventDemo!SetImageWindow(1,42)
  Menu {Tools}{Info}{Structure} Select table()
  EventDemo!SetImageWindow(7,1)
EndProc ; EventDemo!OpenImages

Proc EventDemo!SetImageWindow(WRow,WCol)
   Private
       WindowHandle,
       WindowBag
   WindowHandle = GetWindow()
   Dynarray WindowBag[]
   WindowBag["Width"] = 35
   WindowBag["Height"] = 5
   WindowBag["OriginRow"] = WRow
   WindowBag["OriginCol"] = WCol
   Window SetAttributes WindowHandle From WindowBag
EndProc ; EventDemo!SetImageWindow

;--------------------------- DRAG & DROP WINDOW -----------------------------

Proc EventDemo!OpenDragCanvas()
   Private
      WindowHandle,
      WindowBag
   Window Create @9,31 Height 5 Width 10 to WindowHandle
   SetCanvas WindowHandle
   ?? "Click & drag"
   ? "here for drag"
   ? "& drop demo."
   Dynarray EventDemo_CanvasProcBag[]
   EventDemo_CanvasProcBag["LeftDown"] = "EventDemo!StartDragAndDrop"
   EventDemo_CanvasProcBag["LeftMove"] = "EventDemo!DrawFrame"
   EventDemo_CanvasProcBag["Up"] = "EventDemo!EndDragAndDrop"
   EventDemo_CanvasProcBag["ObjectTag"] = WindowHandle
   EventMan.SetHandlersFrom(EventDemo_CanvasProcBag)
EndProc ; EventDemo!OpenDragCanvas

;----------------------------------------------------------------------------
; This drag and drop demo is modal by virtue of the fact that 
; EventMan.CompleteMouseSequence() is called to complete the mouse event. That
; procedure throws away all but MOUSE UP, MOVE, and AUTO events. It 
; returns 1, so that none of the mouse events are passed back to the calling
; wait or getevent loop
;----------------------------------------------------------------------------
Proc EventDemo!StartDragAndDrop()
   Private
      MRow,
      MCol,
      HitRow
   MRow = EventMan.EventBag["Row"]
   MCol = EventMan.EventBag["Col"]
   LocalizeEvent EventMan.EventBag
   HitRow = EventMan.EventBag["Row"]
   EventMan.EventBag["Row"] = MRow
   EventMan.EventBag["Col"] = MCol
   If HitRow = 0 Then
      EventDemo_IsDragAndDrop = False
      Return 0
   Else
      EventDemo_IsDragAndDrop = True
      Echo Normal Echo off
      EventDemo!DrawFrame()
      EventMan.CompleteMouseSequence()
      Return EventMan.ReturnCode
   Endif
EndProc ; EventDemo!StartDragAndDrop

Proc EventDemo!DrawFrame()
   Private
      MRow,
      MCol
   If EventDemo_IsDragAndDrop Then
      SetCanvas Default
      Canvas off
      Echo Normal Echo off
      MRow = Min(EventMan.EventBag["Row"],21)
      MCol = Min(EventMan.EventBag["Col"],73)
      Frame Single From MRow, MCol to MRow + 3, MCol + 6
      Canvas On
      Return 1
   Else
      Return 0
   Endif
EndProc ; EventDemo!DrawFrame

Proc EventDemo!EndDragAndDrop()
   Private
      MRow,
      MCol
   If EventDemo_IsDragAndDrop Then
      SetCanvas Default
      Canvas off
      Echo Normal Echo off
      MRow = Min(EventMan.EventBag["Row"],21)
      MCol = Min(EventMan.EventBag["Col"],73)
      PaintCanvas Attribute 64 MRow,MCol,MRow + 3,MCol + 6
      Canvas On
      Beep
      Sleep 250
      Beep
      Echo Normal
      EventDemo_IsDragAndDrop = False
      Return 1
   Else
      If EventMan.MouseModeButton = "RIGHT" Then
         Return EventDemo!ControlMenu()
      Else
         Return 0
      Endif
   Endif
EndProc ; EventDemo!EndDragAndDrop

; -------------------------- DEFAULT PROCS ----------------------------------

Proc EventDemo!SetDefaultProcs()
   Dynarray EventDemo_DefaultProcBag[]
   EventDemo_DefaultProcBag[Asc("CoEditKey")] = "EventDemo!SetCoEdit"
   EventDemo_DefaultProcBag[Asc("EditKey")] = "EventDemo!SetEdit"
   EventDemo_DefaultProcBag[Asc("Do_It!")] = "EventDemo!EndEdit"
   EventDemo_DefaultProcBag["RightDown"] = "EventDemo!HandleRightDown"
   EventDemo_DefaultProcBag["Up"] = "EventDemo!CallControlMenu"
   EventDemo_DefaultProcBag[-50] = "EventDemo!ControlMenu" ; AltM
   EventDemo_DefaultProcBag["MenuTag:DemoMenu"] = "EventDemo!ControlMenu"
   EventDemo_DefaultProcBag["MenuTag:Walking"] = "EventDemo!Walking"
   EventDemo_DefaultProcBag["MenuTag:ClearPulldown"] = "EventDemo!ClearPulldown"
;   EventDemo_DefaultProcBag[Asc("Dos")] = "EventMan.DisableEvent"
;   EventDemo_DefaultProcBag[Asc("DosBig")] = "EventMan.DisableEvent"
   EventMan.SetHandlersFrom(EventDemo_DefaultProcBag)
EndProc ; EventDemo!SetDefaultProcs

Proc EventDemo!HandleRightDown()
   If WindowAt(EventMan.EventBag["Row"],EventMan.EventBag["Col"]) <>
      GetWindow() Then
      ExecEvent EventMan.EventBag
   Endif
   Return EventMan.CompleteMouseSequence()
EndProc

Proc EventDemo!SetCoEdit()
   If IsAssigned(EventMan.isWait) Then 
      Message "Setting COEDIT programmatically..."
      CoEditKey
      Return 1
   Else
      Return 0
   Endif
EndProc ; EventDemo!SetCoEdit

Proc EventDemo!SetEdit()
   If IsAssigned(EventMan.isWait) Then 
      Message "Setting EDIT programmatically"
      EditKey
      Return 1
   Else
      Return 0
   Endif
EndProc ; EventDemo!SetEdit

Proc EventDemo!EndEdit()
   If IsAssigned(EventMan.IsWait) And
      Sysmode() = "CoEdit" Or Sysmode() = "Edit" Then
      Message "Ending EDIT or COEDIT programmatically..."
      Do_It!
      Return 1
   Else
      Return 0
   Endif
EndProc ; EventDemo!EndEdit

Proc EventDemo!Walking()
   If Not IsAssigned(WalkingMan.IsActive) Then
      WalkingMan.Constructor()
      Dynarray EventDemo_WalkingProcBag[]
      EventDemo_WalkingProcBag["Close"] = "EventDemo!Walking"
      EventDemo_WalkingProcBag[Asc("WinResize")] = "WalkingMan.Resize"
      EventDemo_WalkingProcBag["LeftDown"] = "WalkingMan.HandleMouse"
      EventDemo_WalkingProcBag["LeftDoubleDown"] = "WalkingMan.HandleMouse"
      EventDemo_WalkingProcBag["ObjectTag"] = StrVal(WalkingMan.WindowHandle)
      EventMan.SetHandlersFrom(EventDemo_WalkingProcBag)
      EventMan.RegisterIdleServer("Walking","WalkingMan.Move")
   Else
      EventMan.DeRegisterIdleServer("Walking")
      EventMan.ClearHandlersWith(EventDemo_WalkingProcBag)
      WalkingMan.Destructor()
   Endif
EndProc ; EventDemo!Walking

; ------------------------------ CONTROL PROCEDURES -------------------------

Proc EventDemo.DispatchEventTag()
   If EventMan.TargetWindow = EventDemo_HelpWindowHandle And
      EventMan.EventType = "KEY" Then
      EventMan.EventTag = "Key"
   Endif
   EventMan.DispatchEventTag()
EndProc ; EventDemo.DispatchEventTag

Proc EventDemo!CloseApp()
   Private
      Command
   ShowPopup "Are you sure?" Centered
      "No":"Do NOT close the demo application":"Cancel",
      "Yes":"Close the demo application":"OK"
   EndMenu
   To Command
   If Command <> "OK" Then
      Return 1
   Else
      If IsAssigned(Tracer.IsActive) Then
         Tracer.Destructor()
      Endif
      EventMan.ClearHandlersWith(EventDemo_DefaultProcBag)
      EventDemo_IsRunning = False
      ClearAll
      While IsWindow(GetWindow())
         Window Close
      EndWhile
      Return 2
   Endif
EndProc ; EventDemo!CloseApp

Proc EventDemo!CallControlMenu()
   If EventMan.MouseModeButton = "RIGHT" Then
      Return EventDemo!ControlMenu()
   Else
      Return 0
   Endif
EndProc

Proc EventDemo!ControlMenu()
   Private
      Command,
      ReturnCode,
      TracerChoice
   Message "Current mode is ",EventDemo_Mode
   While True
      If IsAssigned(Tracer.IsActive) Then
         TracerChoice = "Tr~a~cer menu"
      Else
         TracerChoice = "Load Tr~a~cer"
      Endif
      ShowPopup "Event demo" Centered
         "Wait ~W~orkspace":"Change to wait workspace mode":"Workspace",
         "Wait ~T~able":"Change to wait table mode":"Table",
         "Wait ~R~ecord":"Change to wait table mode":"Record",
         "Wait ~F~ield":"Change to wait field mode":"Field",
         "~G~etEvent":"Change to GetEvent mode":"GetEvent",
         Separator,
         TracerChoice:"Show tracer menu":"Trace",
         "Show ~h~elp":"Show help":"Help",
         "Set ~M~enubar":"Set menu bar":"SetPulldown",
         Separator,
         "~Q~uit demo":"Quit the event demo application":"Quit"
      EndMenu
      To Command
      If Not ((Command = "Workspace" or Command = "Table" or Command = "Record" or
         Command = "Field") And (nImages() = 0)) Then
         Quitloop
      Else
         Message "Cannot enter wait loop with no table images."
      Endif
   EndWhile
   Switch
      Case Command = "Esc":
         ReturnCode = 1
      Case Command = "Trace":
         If IsAssigned(Tracer.IsActive) Then
            Tracer.ControlMenu()
         Else
            Tracer.Constructor()
            Tracer.OpenEventTrace()
         Endif
         ReturnCode = 1
      Case Command = "GetEvent":
         If Not EventDemo_Mode = "GetEvent" Then
            EventDemo_Mode = Command
            ReturnCode = 2
            Message "Changed to GETEVENT loop..."
         Else
            Message "Already in GetEvent loop..."
            ReturnCode = 1
         Endif
      Case Command = "Workspace":
         If Not EventDemo_Mode = "Workspace" Then
            EventDemo_Mode = Command
            ReturnCode = 2
            Message "Changed to WAIT WORKSPACE loop..."
         Else
            Message "Already in Wait Workspace loop..."
            ReturnCode = 1
         Endif
      Case Command = "Table":
         If Not EventDemo_Mode = "Table" Then
            EventDemo_Mode = Command
            ReturnCode = 2
            Message "Changed to WAIT TABLE loop..."
         Else
            Message "Already in Wait Table loop..."
            ReturnCode = 1
         Endif
      Case Command = "Record":
         If Not EventDemo_Mode = "Record" Then
            EventDemo_Mode = Command
            ReturnCode = 2
            Message "Changed to WAIT RECORD loop..."
         Else
            Message "Already in Wait Record loop..."
            ReturnCode = 1
         Endif
      Case Command = "Field":
         If Not EventDemo_Mode = "Field" Then
            EventDemo_Mode = Command
            ReturnCode = 2
            Message "Changed to WAIT FIELD loop..."
         Else
            Message "Already in Wait Field loop..."
            ReturnCode = 1
         Endif
      Case Command = "Help":
         If Not EventDemo_HelpWindowIsActive Then
            Echo Off
            EventDemo!OpenEventHelp()
            Echo Normal
         Else
            Window Select EventDemo_HelpWindowHandle
            Message "Event HELP window already on desktop..."
         Endif
         ReturnCode = 1
      Case Command = "Quit":
         Message "Quitting."
         ReturnCode = EventDemo!CloseApp()
         If ReturnCode = 2 Then
            Message "Event demo is over."
         Else
            Message ""
         Endif
       Case Command = "SetPulldown":
          EventDemo!SetPulldown()
          ReturnCode = 1
   EndSwitch
   Return ReturnCode
EndProc ; EventDemo!ControlMenu

Proc EventDemo!SetPullDown()
   ShowPullDown
      "~D~emoMenu":"Show the demo popup menu.":"DemoMenu",
      "~W~alking":"Toggle walking man":"Walking",
      "Clear~M~enubar":"Clear this pulldown menu to show the Paradox menu.":
          "ClearPulldown"
   EndMenu
EndProc

Proc EventDemo!ClearPullDown()
   ClearPulldown
EndProc

; ----------------------------------- MAINLINE ------------------------------

If Not IsAssigned(Librarian.HasControl) Then

   Proc Closed EventDemo!DoDemo()
      ClearAll
      While IsWindow(GetWindow())
         Window Close
      EndWhile
      EventDemo.Constructor()
      EventDemo.Run()
      EventDemo.Destructor()
   EndProc

   EventDemo!DoDemo()

Endif
